# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import wx
import wx.grid
import coda
import numpy

import sys
import math

from . import frame
from . import model

usePlotRenderer = 1
try:
    from visan.plot.plotwindow import PlotWindow
except ImportError:
    usePlotRenderer = 0


# EXCEPTION HIERARCHY
class RenderError(Exception):
    """Generalization of all rendering errors"""
    pass


# CUSTOM EVENTS
wxEVT_SLICE_CHANGED = wx.NewEventType()
EVT_SLICE_CHANGED = wx.PyEventBinder(wxEVT_SLICE_CHANGED)


class SliceChangedEvent(wx.PyCommandEvent):

    def __init__(self, id):
        wx.PyCommandEvent.__init__(self, wxEVT_SLICE_CHANGED, id)


# UTILITY FUNCTIONS
def GetDecimalWidth(dc, number, template="%s"):
    assert number >= 0, "No negative numbers allowed."

    if number == 0:
        digitCount = 1
    else:
        digitCount = int(math.log10(number)) + 1

    tmp = template % ("9" * digitCount,)
    return dc.GetTextExtent(tmp)[0]


class NumpySlicer1D(wx.Panel):

    def __init__(self, parent, id, array, axisLabel):
        wx.Panel.__init__(self, parent, id)

        assert array.ndim >= 2, "Expected numpy array of at least rank 2."

        # this corrects the colour of the panel in win32 (without this
        # line it defaults to dark grey, instead of light grey). taken
        # from: PlotFrame.py
        self.SetBackgroundColour(self.GetBackgroundColour())

        self.array = array
        self.spinMap = {}
        self.spinners = []
        self.index = [0] * self.array.ndim

        # create UI
        self.axisGroup = frame.ButtonGroup()

        # compute button dimensions
        buttonWidth, buttonHeight = self.GetTextExtent(axisLabel)

        # compute spinner width
        spinnerWidth = 0
        for i in range(0, self.array.ndim):
            spinnerWidth = max(spinnerWidth, GetDecimalWidth(self, self.array.shape[i] - 1))

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add((0, 0), 1)
        for i in range(0, self.array.ndim):
            spinControl = wx.SpinCtrl(self, -1, style=wx.SP_ARROW_KEYS | wx.TE_PROCESS_ENTER)
            spinControl.SetRange(0, self.array.shape[i] - 1)
            spinControl.SetValue(0)
            spinControl.SetSizeHints(spinnerWidth + 30, -1)
            self.spinMap[spinControl.GetId()] = i
            self.spinners.append(spinControl)

            button = wx.ToggleButton(self, -1, axisLabel)
            button.SetSizeHints(buttonWidth + 10, -1)
            self.axisGroup.AppendButton(button)

            axisSizer = wx.BoxSizer(wx.VERTICAL)
            axisSizer.Add(spinControl, 0, wx.EXPAND, 0)
            axisSizer.Add((3, 3), 0, 0, 0)
            axisSizer.Add(button, 0, wx.EXPAND, 0)
            sizer.Add(axisSizer, 0, wx.ALIGN_CENTER | wx.ALL, 3)
        sizer.Add((0, 0), 1)
        self.SetSizer(sizer)

        self.spinners[-1].Disable()
        self.axisGroup.SetSelected(self.array.ndim - 1)
        self.sliceAxis = self.array.ndim - 1
        self.activeAxes = [x for x in range(0, self.array.ndim) if x != self.sliceAxis]

        self.Bind(wx.EVT_TOGGLEBUTTON, self.OnToggleButton)
        self.Bind(wx.EVT_SPINCTRL, self.OnSpinCtrl)
        self.Bind(wx.EVT_TEXT_ENTER, self.OnSpinCtrlText)

    def _GetIndexRange(self, dim):
        if dim in self.activeAxes:
            return self.index[dim]
        else:
            return slice(None)

    def GetSlice(self):
        return self.array[tuple([self._GetIndexRange(x) for x in range(self.array.ndim)])]

    def _SetSliceAxis(self, axis):
        self.sliceAxis = axis
        self.activeAxes = [x for x in range(self.array.ndim) if x != self.sliceAxis]

        wx.PostEvent(self, SliceChangedEvent(self.GetId()))
        if wx.Platform == '__WXMAC__':
            wx.Yield()

    def OnToggleButton(self, event):
        prevAxis = self.axisGroup.GetSelected()
        if self.axisGroup.ProcessToggleEvent(event.GetEventObject()):
            currentAxis = self.axisGroup.GetSelected()
            self.spinners[prevAxis].Enable()
            self.spinners[currentAxis].Disable()
            self._SetSliceAxis(currentAxis)

    def OnSpinCtrl(self, event):
        spinner = event.GetEventObject()

        try:
            axis = self.spinMap[event.GetId()]
        except KeyError:
            raise frame.ProductBrowserError("Event received from unknown SpinCtrl (%i,%i)" %
                                            (self.GetId(), event.GetId()))

        spinner.SetValue(event.GetPosition())
        self.index[axis] = spinner.GetValue()

        wx.PostEvent(self, SliceChangedEvent(self.GetId()))
        if wx.Platform == '__WXMAC__':
            wx.Yield()

    def OnSpinCtrlText(self, event):
        spinner = event.GetEventObject()

        try:
            axis = self.spinMap[event.GetId()]
        except KeyError:
            raise frame.ProductBrowserError("Event received from unknown SpinCtrl (%i,%i)" %
                                            (self.GetId(), event.GetId()))

        spinner.SetValue(int(event.GetString()))
        self.index[axis] = spinner.GetValue()

        wx.PostEvent(self, SliceChangedEvent(self.GetId()))
        if wx.Platform == '__WXMAC__':
            wx.Yield()


class NumpySlicer2D(wx.Panel):

    def __init__(self, parent, id, array, axisLabels):
        wx.Panel.__init__(self, parent, id)

        assert array.ndim >= 3, "Expected numpy array of at least rank 3."

        # this corrects the colour of the panel in win32 (without this
        # line it defaults to dark grey, instead of light grey). taken
        # from: PlotFrame.py
        self.SetBackgroundColour(self.GetBackgroundColour())

        self.array = array
        self.spinMap = {}
        self.spinners = []
        self.axisGroups = []
        self.index = [0] * self.array.ndim

        # create UI
        self.axisGroups.append(frame.ButtonGroup())
        self.axisGroups.append(frame.ButtonGroup())

        # compute button dimensions
        buttonWidth = 0
        for i in range(0, 2):
            buttonWidth = max(buttonWidth, self.GetTextExtent(axisLabels[i])[0])

        # compute spinner width
        spinnerWidth = 0
        for i in range(0, self.array.ndim):
            spinnerWidth = max(spinnerWidth, GetDecimalWidth(self, self.array.shape[i] - 1))

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add((0, 0), 1)
        for i in range(0, self.array.ndim):
            spinControl = wx.SpinCtrl(self, -1, style=wx.SP_ARROW_KEYS | wx.TE_PROCESS_ENTER)
            spinControl.SetRange(0, self.array.shape[i] - 1)
            spinControl.SetValue(0)
            spinControl.SetSizeHints(spinnerWidth + 30, -1)
            self.spinMap[spinControl.GetId()] = i
            self.spinners.append(spinControl)

            buttonSizer = wx.BoxSizer(wx.HORIZONTAL)

            # slow dimension (0)
            button = wx.ToggleButton(self, -1, axisLabels[0])
            button.SetSizeHints(buttonWidth + 10, -1)
            self.axisGroups[0].AppendButton(button)
            buttonSizer.Add(button, 1, 0, 0)

            buttonSizer.Add((3, 3), 0, 0, 0)

            # fast dimension (1)
            button = wx.ToggleButton(self, -1, axisLabels[1])
            button.SetSizeHints(buttonWidth + 10, -1)
            self.axisGroups[1].AppendButton(button)
            buttonSizer.Add(button, 1, 0, 0)

            axisSizer = wx.BoxSizer(wx.VERTICAL)
            axisSizer.Add(spinControl, 0, wx.EXPAND, 0)
            axisSizer.Add((3, 3), 0, 0, 0)
            axisSizer.Add(buttonSizer, 0, wx.EXPAND, 0)
            sizer.Add(axisSizer, 0, wx.ALIGN_CENTER | wx.ALL, 3)

        sizer.Add((0, 0), 1)
        self.SetSizer(sizer)

        # one dimension before the fastest dimension
        self.spinners[-2].Disable()
        self.axisGroups[0].SetSelected(self.array.ndim - 2)
        # fastest dimension
        self.spinners[-1].Disable()
        self.axisGroups[1].SetSelected(self.array.ndim - 1)

        self.sliceAxes = [self.array.ndim - 2, self.array.ndim - 1]
        self.activeAxes = [x for x in range(0, self.array.ndim) if x not in self.sliceAxes]

        self.Bind(wx.EVT_TOGGLEBUTTON, self.OnToggleButton)
        self.Bind(wx.EVT_SPINCTRL, self.OnSpinCtrl)
        self.Bind(wx.EVT_TEXT_ENTER, self.OnSpinCtrlText)

    def _GetIndexRange(self, dim):
        if dim in self.activeAxes:
            return self.index[dim]
        else:
            return slice(None)

    def GetSlice(self):
        return self.array[tuple(self._GetIndexRange(x) for x in range(self.array.ndim))]

    def _SetSliceAxes(self, sliceAxis, axis):
        self.sliceAxes[sliceAxis] = axis
        self.activeAxes = [x for x in range(0, self.array.ndim) if x not in self.sliceAxes]

        wx.PostEvent(self, SliceChangedEvent(self.GetId()))
        if wx.Platform == '__WXMAC__':
            wx.Yield()

    def _TransposeSliceAxes(self):
        self.sliceAxes.reverse()

        wx.PostEvent(self, SliceChangedEvent(self.GetId()))
        if wx.Platform == '__WXMAC__':
            wx.Yield()

    def OnToggleButton(self, event):
        currentSlow = self.axisGroups[0].GetSelected()
        currentFast = self.axisGroups[1].GetSelected()

        if self.axisGroups[0].ProcessToggleEvent(event.GetEventObject()):
            newSlow = self.axisGroups[0].GetSelected()

            if newSlow == currentFast:
                self.axisGroups[0].SetSelected(newSlow)
                self.axisGroups[1].SetSelected(currentSlow)
                self._TransposeSliceAxes()
            else:
                self.spinners[currentSlow].Enable()
                self.spinners[newSlow].Disable()
                self._SetSliceAxes(0, newSlow)

        elif self.axisGroups[1].ProcessToggleEvent(event.GetEventObject()):
            newFast = self.axisGroups[1].GetSelected()

            if newFast == currentSlow:
                self.axisGroups[0].SetSelected(currentFast)
                self.axisGroups[1].SetSelected(newFast)
                self._TransposeSliceAxes()
            else:
                self.spinners[currentFast].Enable()
                self.spinners[newFast].Disable()
                self._SetSliceAxes(1, newFast)

    def OnSpinCtrl(self, event):
        spinner = event.GetEventObject()

        try:
            axis = self.spinMap[event.GetId()]
        except KeyError:
            raise frame.ProductBrowserError("Event received from unknown SpinCtrl (%i,%i)" %
                                            (self.GetId(), event.GetId()))

        spinner.SetValue(event.GetPosition())
        self.index[axis] = spinner.GetValue()

        wx.PostEvent(self, SliceChangedEvent(self.GetId()))
        if wx.Platform == '__WXMAC__':
            wx.Yield()

    def OnSpinCtrlText(self, event):
        spinner = event.GetEventObject()

        try:
            axis = self.spinMap[event.GetId()]
        except KeyError:
            raise frame.ProductBrowserError("Event received from unknown SpinCtrl (%i,%i)" %
                                            (self.GetId(), event.GetId()))

        spinner.SetValue(int(event.GetString()))
        self.index[axis] = spinner.GetValue()

        wx.PostEvent(self, SliceChangedEvent(self.GetId()))
        if wx.Platform == '__WXMAC__':
            wx.Yield()



class NumpyWrapper(wx.grid.GridTableBase):

    def __init__(self, base, array):
        wx.grid.GridTableBase.__init__(self)

        self.base = base
        self.array = array

    def GetAttr(self, row, col, kind):
        tmp = wx.grid.GridCellAttr()
        tmp.SetReadOnly(True)
        return tmp

    def GetNumberRows(self):
        return self.array.shape[0]

    def GetNumberCols(self):
        return self.array.shape[1]

    def GetColLabelValue(self, col):
        return str(col)

    def GetRowLabelValue(self, row):
        return str(row)

    def IsEmptyCell(self, row, col):
        return False

    def GetValue(self, row, col):
        return model.GetDataAsString(self.base, self.array[row, col])

    def SetValue(self, row, col, value):
        pass


class HexNumpyWrapper(wx.grid.GridTableBase):

    def __init__(self, array):
        assert (array.ndim == 1), "only arrays of rank one are supported"
        assert array.size > 0, "array should not be empty"

        wx.grid.GridTableBase.__init__(self)
        self.array = array

        self.rowCount = int(math.ceil(float(array.size) / 16.0))
        self.rowLabelSize = int(math.ceil(math.log10(array.size) / math.log10(16)))

    def GetAttr(self, row, col, kind):
        tmp = wx.grid.GridCellAttr()
        tmp.SetReadOnly(True)
        return tmp

    def GetNumberRows(self):
        return self.rowCount

    def GetNumberCols(self):
        return 16

    def GetColLabelValue(self, col):
        return "%.1X" % (col,)

    def GetRowLabelValue(self, row):
        value = "%X" % (row * 16,)
        padding = "0" * (self.rowLabelSize - len(value))
        return padding + value

    def IsEmptyCell(self, row, col):
        return ((row * 16 + col) >= self.array.size)

    def GetValue(self, row, col):
        # why do we have to check for empty cells here?
        # on would expect the wx.Grid class to first check if a
        # cell is empty, and only call GetValue() if it is not.
        if self.IsEmptyCell(row, col):
            return ""
        else:
            return "%.2X" % (self.array[row * 16 + col],)

    def SetValue(self, row, col, value):
        pass


class ArrayRenderer(wx.Panel):

    # class method that can examine a node to make sure
    # it can be rendered by this renderer.
    def CanRender(cls, node):
        return (not isinstance(node, model.ObjectArrayNode)) and (len(node) > 0)
    CanRender = classmethod(CanRender)

    name = "2D Grid"

    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)

        # this corrects the colour of the panel in win32 (without this
        # line it defaults to dark grey, instead of light grey). taken
        # from: PlotFrame.py
        self.SetBackgroundColour(self.GetBackgroundColour())

    def Initialize(self, node):
        assert node.type == model.TYPE_ARRAY, "ArrayRenderer can only render node of type TYPE_ARRAY"
        assert not isinstance(node, model.ObjectArrayNode), "ArrayRenderer cannot render arrays of compound types"

        # coda.fetch() will promote a rank-0 array to a 1x1 array,
        # so a rank of 1 is the smallest possible rank.
        try:
            self.array = coda.fetch(node.cursor)
        except (coda.CodaError, coda.CodacError) as ex:
            raise RenderError("[CODA] %s" % (str(ex),))

        # array sanity check
        ok = True
        if node.isRankZero():
            ok = (self.array.ndim == 1) and (self.array.shape[0] == 1)
        else:
            ok = (self.array.ndim == len(node.dimensions)) and (self.array.size == len(node))
            i = 0
            while ok and i < len(node.dimensions):
                ok = ok and (self.array.shape[i] == node.dimensions[i])
                i += 1

        if not ok:
            raise RenderError("[ArrayRenderer] Array read from product does not match description.")

        self.base = node.base
        self.slice = None

        # coda.fetch() will promote a rank-0 array to a 1x1 array,
        # so a rank of 1 is the smallest possible rank.
        if self.array.ndim == 1:
            self.array = self.array[:, numpy.newaxis]

        self.valueText = wx.TextCtrl(self, -1, "")
        valueSizer = wx.BoxSizer(wx.HORIZONTAL)
        valueSizer.Add(wx.StaticText(self, -1, "Value:"), 0, wx.ALIGN_CENTER, 0)
        valueSizer.Add((5, 5), 0, wx.EXPAND)
        valueSizer.Add(self.valueText, 1, 0, 0)

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.grid = wx.grid.Grid(self, -1, style=wx.BORDER_SUNKEN)

        # TODO: use wxGrid::SetDefaultColSize() and row equiv. as an approx.
        # of AutoSize().
        if self.array.ndim > 2:
            self.slicer = NumpySlicer2D(self, -1, self.array, ("row", "col"))
            self.Bind(EVT_SLICE_CHANGED, self.OnSliceChanged)

        self.UpdateGridData()
        # _really_ slow, so only used for string data (which can require a large cell width).
        if self.base == model.TYPE_STRING:
            self.grid.AutoSize()

        self.sizer.Add(valueSizer, 0, wx.EXPAND | wx.ALL, 5)
        self.sizer.Add(wx.StaticLine(self, -1, style=wx.LI_HORIZONTAL), 0, wx.EXPAND | wx.LEFT | wx.RIGHT, 5)
        self.sizer.Add(self.grid, 1, wx.EXPAND | wx.ALL, 5)
        if self.array.ndim > 2:
            self.sizer.Add(self.slicer, 0, wx.EXPAND | wx.LEFT | wx.RIGHT, 5)
        self.SetSizer(self.sizer)

        self.Bind(wx.grid.EVT_GRID_SELECT_CELL, self.OnSelectCell)

    def UpdateGridData(self):
        if self.array.ndim <= 2:
            self.slice = self.array
        else:
            self.slice = self.slicer.GetSlice()

        self.grid.SetTable(NumpyWrapper(self.base, self.slice), True)
        self.valueText.SetValue(model.GetDataAsString(self.base, self.slice.flat[0], 256))

    def OnSliceChanged(self, event):
        self.sizer.Detach(self.grid)
        self.grid.Destroy()

        self.grid = wx.grid.Grid(self, -1)
        self.UpdateGridData()
        # _really_ slow...
        if self.base == model.TYPE_STRING:
            self.grid.AutoSize()

        self.sizer.Insert(2, self.grid, 1, wx.EXPAND | wx.ALL, 5)
        self.sizer.Layout()

    def OnSelectCell(self, event):
        if event.Selecting():
            self.valueText.SetValue(model.GetDataAsString(self.base, self.slice[event.GetRow(), event.GetCol()], 256))
        else:
            self.valueText.SetValue("")
        event.Skip()


class PlotRenderer(wx.Panel):

    # class method that can examine a node to make sure
    # it can be rendered by this renderer.
    plotableTypes = (model.TYPE_INT8, model.TYPE_UINT8, model.TYPE_INT16, model.TYPE_UINT16, model.TYPE_INT32,
                     model.TYPE_UINT32, model.TYPE_INT64, model.TYPE_UINT64, model.TYPE_VSF_INTEGER,
                     model.TYPE_FLOAT32, model.TYPE_FLOAT64)

    def CanRender(cls, node):
        return (usePlotRenderer and (not isinstance(node, model.ObjectArrayNode)) and
                (node.base in PlotRenderer.plotableTypes) and (len(node) > 0))
    CanRender = classmethod(CanRender)

    name = "2D Plot"

    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)
        self.SetBackgroundColour(self.GetBackgroundColour())  # Set the background 'explicitly' to have it stick

    def Initialize(self, node):
        assert node.type == model.TYPE_ARRAY, "PlotRenderer can only render node of type TYPE_ARRAY"
        assert not isinstance(node, model.ObjectArrayNode), "PlotRenderer cannot render arrays of compound types"

        # coda.fetch() will promote a rank-0 array to a 1x1 array,
        # so a rank of 1 is the smallest possible rank.
        try:
            self.array = coda.fetch(node.cursor)
        except (coda.CodaError, coda.CodacError) as ex:
            raise RenderError("[CODA] %s" % (str(ex),))

        # array sanity check
        ok = True
        if node.isRankZero():
            ok = (self.array.ndim == 1) and (self.array.shape[0] == 1)
        else:
            ok = (self.array.ndim == len(node.dimensions)) and (self.array.size == len(node))
            i = 0
            while ok and i < len(node.dimensions):
                ok = ok and (self.array.shape[i] == node.dimensions[i])
                i += 1

        if not ok:
            raise RenderError("[PlotRenderer] Array read from product does not match description.")

        self.plot = PlotWindow(self, -1)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.plot, 1, wx.EXPAND | wx.ALL, 5)
        if self.array.ndim > 1:
            self.slicer = NumpySlicer1D(self, -1, self.array, "select")
            sizer.Add(self.slicer, 0, wx.EXPAND | wx.LEFT | wx.RIGHT, 5)
            self.Bind(EVT_SLICE_CHANGED, self.OnSliceChanged)

        self.SetSizer(sizer)

        if self.array.ndim == 1:
            self.plot.AddDataSet(numpy.arange(self.array.shape[0]), self.array)
        else:
            slice = self.slicer.GetSlice()
            self.plot.AddDataSet(numpy.arange(slice.shape[0]), slice)

    def UpdatePlotData(self):
        if self.array.ndim > 1:
            slice = self.slicer.GetSlice()
            self.plot.UpdateDataSet(0, numpy.arange(slice.shape[0]), slice)
            # update x-axis and y-axis ranges
            self.plot.SetAxisRange(0, 0, slice.shape[0] - 1)
            self.plot.SetAxisRange(1, numpy.nanmin(slice), numpy.nanmax(slice))

    def OnSliceChanged(self, event):
        self.UpdatePlotData()


class RecordRenderer(wx.ListCtrl):

    # class method that can examine a node to make sure
    # it can be rendered by this renderer.
    def CanRender(cls, node):
        return not node.dirty() and len(node) > 0
    CanRender = classmethod(CanRender)

    name = "List"

    def __init__(self, parent):
        style = wx.LC_REPORT | wx.BORDER_SUNKEN | wx.LC_VRULES | wx.LC_SINGLE_SEL | wx.FULL_REPAINT_ON_RESIZE
        wx.ListCtrl.__init__(self, parent, -1, style=style)

        self.SetImageList(model.iconImageList, wx.IMAGE_LIST_SMALL)
        self.InsertColumn(0, "ID", wx.LIST_FORMAT_RIGHT)
        self.InsertColumn(1, "Name")
        self.InsertColumn(2, "Value")

    def Initialize(self, node):
        for i in range(0, len(node)):
            field = node[i]

            if not (coda.get_option_filter_record_fields() and (field.hidden or not field.available)):
                idx = self.InsertItem(i, str(i))
                self.SetItem(idx, 1, field.real_name, model.GetNodeIcon(field, model.iconDictionary))

                if field.available:
                    if field.type == model.TYPE_RECORD:
                        self.SetItem(idx, 2, "<record of %u field(s)>" % (len(field),))

                    elif field.type == model.TYPE_ARRAY:
                        if field.isRankZero():
                            self.SetItem(idx, 2, "<rank-0 array of %s>" % (model.GetTypeAsString(field.base),))

                        else:
                            dimensionString = ""
                            for dim in field.dimensions[:-1]:
                                dimensionString += "%u x " % (dim,)
                            dimensionString += "%u" % (field.dimensions[-1],)

                            self.SetItem(idx, 2, "[%s %s]" % (dimensionString, model.GetTypeAsString(field.base)))

                    else:
                        # a maximum size of _maxLineLength characters is specified, because
                        # GDK may crash if the length of a string item gets too long.
                        # also, if the line is really long the user will probably not want
                        # to view as a single line in a ListCtrl anyway.
                        try:
                            data = coda.fetch(field.cursor)
                        except (coda.CodaError, coda.CodacError) as ex:
                            raise RenderError("[CODA] %s" % (str(ex),))

                        self.SetItem(idx, 2, model.GetDataAsString(field.type, data, frame._maxLineLength))
                else:
                    self.SetItem(idx, 2, "<unavailable>")

        # _AutoSize() sets the width of a column to the maximum of the
        # width of the longest item in the column and the width of the column
        # header.
        # I would like to _AutoSize() only column 0 and 1, and have column 2
        # take up the rest of the available space (maybe constrained to a
        # minimum width, i.e. the width of the smallest item in the column).
        # This would be desirable, because sometimes there can be very long
        # items, which result in very small scrollbars. However, there does not
        # seem to be an easy way to achieve this. (Note that if an item is
        # longer than the column width, it is automatically clipped and
        # padded with '...' by the ListCtrl.)
        #
        # UPDATE: A specified maximum length (productbrowser.frame._maxLength) is
        # now used to avoid very long items. Therefore, we can now _AutoSize()
        # column 2 as well. However, _AutoSize()-ing only column 0 and 1, and
        # having column 2 take up the rest of the available space would still
        # be a better solution.
        self._AutoSize((0, 1, 2))

    def _AutoSize(self, columns):
        for col in columns:
            header = self.GetColumn(col)
            if header is not None:
                self.SetColumnWidth(col, wx.LIST_AUTOSIZE)
                columnWidth = self.GetColumnWidth(col)
                headerWidth, headerHeight = self.GetTextExtent(header.GetText())
                self.SetColumnWidth(col, max(columnWidth, headerWidth + 15))
                if wx.Platform == '__WXMSW__' and col == 0:
                    self.SetColumnWidth(0, self.GetColumnWidth(0) + 15)


class BytesRenderer(wx.grid.Grid):

    # class method that can examine a node to make sure
    # it can be rendered by this renderer.
    def CanRender(cls, node):
        return True
    CanRender = classmethod(CanRender)

    name = "Hex"

    def __init__(self, parent):
        wx.grid.Grid.__init__(self, parent, -1, style=wx.BORDER_SUNKEN)

    def Initialize(self, node):
        try:
            array = coda.fetch(node.cursor)
        except (coda.CodaError, coda.CodacError) as ex:
            raise RenderError("[CODA] %s" % (str(ex),))

        # array sanity check
        if not array.ndim == 1:
            raise RenderError("[BytesRenderer] Array read from product does not match description.")

        if array.size == 0:
            raise RenderError("[BytesRenderer] Array read from product is empty.")

        width, height = self.GetTextExtent("FF")
        self.SetDefaultColSize(width + 10)
        self.SetDefaultRowSize(height + 10)
        self.SetDefaultCellAlignment(wx.ALIGN_CENTRE, wx.ALIGN_CENTRE)
        self.SetTable(HexNumpyWrapper(array), True)


class ScalarRenderer(wx.TextCtrl):

    # class method that can examine a node to make sure
    # it can be rendered by this renderer.
    def CanRender(cls, node):
        return True
    CanRender = classmethod(CanRender)

    name = "String"

    def __init__(self, parent):
        wx.TextCtrl.__init__(self, parent, -1, "", style=wx.TE_READONLY | wx.TE_MULTILINE)

    def Initialize(self, node):
        try:
            valueString = model.GetDataAsString(node.type, coda.fetch(node.cursor))
        except (coda.CodaError, coda.CodacError) as ex:
            raise RenderError("[CODA] %s" % (str(ex),))

        self.SetValue(valueString)


class UnavailableRenderer(wx.Panel):

    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)
        # this corrects the colour of the panel in win32 (without this
        # line it defaults to dark grey, instead of light grey). taken
        # from: PlotFrame.py
        self.SetBackgroundColour(self.GetBackgroundColour())

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add((0, 0), 1)
        sizer.Add(wx.StaticText(self, -1, "Data unavailable or no renderer associated with this type."), 0,
                  wx.ALIGN_CENTER, 0)
        sizer.Add((0, 0), 1)
        self.SetSizer(sizer)



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""
Startup code for VISAN.
"""

import os
import sys


def main():
    try:
        import traceback
        sys.argv[0] = "visan"
        from .app import VisanApp
        app = VisanApp()
        app.MainLoop()
    except Exception:
        type, value, sys.last_traceback = sys.exc_info()
        lines = traceback.format_exception(type, value, sys.last_traceback)
        text = "Uncaught exception.\n\n" + "".join(lines)
        if sys.platform == 'win32':
            from ctypes import c_int, WINFUNCTYPE, windll
            from ctypes.wintypes import HWND, LPCSTR, UINT
            prototype = WINFUNCTYPE(c_int, HWND, LPCSTR, LPCSTR, UINT)
            paramflags = (1, "hwnd", 0), (1, "text", "Hi"), (1, "caption", None), (1, "flags", 0)
            MessageBox = prototype(("MessageBoxA", windll.user32), paramflags)
            MessageBox(text=text, caption="VISAN Error")
        else:
            sys.__stderr__.write(text)


if __name__ == "__main__":
    main()



# This is an example VISAN script for the MIP_NL__2P product

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing MIP_NL__2P products.

# This example will then take all products it finds in this directory and
# for these products plot the ozone profiles


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all files in productdir starting with 'MIP_NL__2P'
    files = glob.glob(os.path.join(productdir, "MIP_NL__2P*"))
    if len(files) == 0:
        print(("Could not find any MIP_NL__2P files in directory '" + productdir + "'"))
        return

    # When we exclude altitude (but still include pressure), the plot command will use pressure for the y-axis
    # By providing a minimum value for the O3 vmr we also automatically filter out all NaN values
    o3 = harp.import_product(files[0], "exclude(altitude);O3_volume_mixing_ratio>=0", "species=O3")

    window = plot(o3, ylabel="p [hPa]", xmin=0, title="MIP_NL__2P profile example (o3)")


run()



# This is an example VISAN script for the GOME_L2 product

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing GOME L2 products (using ERS binary format).

# This example will then take all L2 products it finds in this directory and
# for these products plot the ozone total columns, overplotted with cloud fractions


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all files in productdir ending with '.lv2'
    files = glob.glob(os.path.join(productdir, "*.lv2"))
    if len(files) == 0:
        print(("Could not find any GOME_L2 files in directory '" + productdir + "'"))
        return

    # Ingest the total ozone column values and the cloud fraction,
    # but ingest only the forward scan pixels and make sure that O3 is in Dobson Units
    data = harp.import_product(files, 'scan_direction_type=="forward";derive(O3_column_number_density [DU])')
    print(data)
    w = wplot(data, colortable="Ozone", colorrange=(150.0, 500.0), numcolorlabels=8)
    wplot(data, value="cloud_fraction", colortable="Cloud", colorrange=(0.0, 1.0), numcolorlabels=5, deltaradius=0.02,
          window=w)
    w.SetPlotTitle('Ozone with cloud cover overlay')


run()



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import numpy
import vtk
from vtk.util.numpy_support import numpy_to_vtk
from .wxVTKRenderWindowInteractor import wxVTKRenderWindowInteractor
import wx

from .visanplot import vtkGeoGridSource, vtkGeoMapFilter, vtkInteractorStyleWorldPlot2D, \
    vtkInteractorStyleWorldPlot3D, vtkProjFilter, vtkColorTable, vtkGeographyLineData, vtkGeoGridData, \
    vtkWorldPlotGridData, vtkWorldPlotLineData, vtkWorldPlotPointData, vtkWorldPlotSwathData

WorldPlotDataChangedEvent, EVT_WORLDPLOTDATA_CHANGED = wx.lib.newevent.NewEvent()
WorldViewChangedEvent, EVT_WORLDVIEW_CHANGED = wx.lib.newevent.NewEvent()

PROJECTION_LAMBERT_CYLINDRICAL = "Lambert Cylindrical"
PROJECTION_PLATE_CAREE = "Plate Caree"
PROJECTION_MOLLWEIDE = "Mollweide"
PROJECTION_ROBINSON = "Robinson"
PROJECTION_LAMBERT_AZIMUTHAL = "Lambert Azimuthal"
PROJECTION_AZIMUTHAL_EQUIDISTANT = "Azimuthal Equidistant"
PROJECTION_3D = "3D"

PROJECTIONS = [PROJECTION_LAMBERT_CYLINDRICAL, PROJECTION_PLATE_CAREE, PROJECTION_MOLLWEIDE, PROJECTION_ROBINSON,
               PROJECTION_LAMBERT_AZIMUTHAL, PROJECTION_AZIMUTHAL_EQUIDISTANT, PROJECTION_3D]
CYLINDRICAL_PROJECTIONS = [PROJECTION_LAMBERT_CYLINDRICAL, PROJECTION_PLATE_CAREE, PROJECTION_MOLLWEIDE,
                           PROJECTION_ROBINSON]
AZIMUTHAL_PROJECTIONS = [PROJECTION_LAMBERT_AZIMUTHAL, PROJECTION_AZIMUTHAL_EQUIDISTANT]

PROJECTION_IDS = {
    PROJECTION_LAMBERT_CYLINDRICAL: 1,
    PROJECTION_PLATE_CAREE: 2,
    PROJECTION_MOLLWEIDE: 3,
    PROJECTION_ROBINSON: 4,
    PROJECTION_LAMBERT_AZIMUTHAL: 5,
    PROJECTION_AZIMUTHAL_EQUIDISTANT: 6,
    PROJECTION_3D: 7,
}

class WorldPlotWindow(wxVTKRenderWindowInteractor):

    def __init__(self, *args, **kwargs):
        super(WorldPlotWindow, self).__init__(*args, **kwargs)

        self.keyframe = 0
        self.numKeyframes = 1
        self.dataSets = []
        self.projCenterLongitude = 0.0
        self.projCenterLatitude = 0.0

        self.colorBarHeight = 60
        self.dataSetForColorBar = -1

        self.renderer2D = vtk.vtkRenderer()
        self.renderer2D.SetBackground(1, 1, 1)

        self.renderer3D = vtk.vtkRenderer()
        self.renderer3D.SetBackground(0, 0, 0)

        self.style2D = vtkInteractorStyleWorldPlot2D()
        self.style3D = vtkInteractorStyleWorldPlot3D()

        # Globe (only applicable for 3D)
        sphere = vtk.vtkSphereSource()
        sphere.SetRadius(1)
        sphere.SetPhiResolution(30)
        sphere.SetThetaResolution(60)

        mapper3D = vtk.vtkPolyDataMapper()
        actor3D = vtk.vtkActor()

        mapper3D.SetInputConnection(sphere.GetOutputPort())
        actor3D.SetMapper(mapper3D)
        self.renderer3D.AddActor(actor3D)

        def onWorldViewChanged(caller, event, calldata=None):
            wx.PostEvent(self, WorldViewChangedEvent())
        self.style2D.AddObserver("WorldViewChanged", onWorldViewChanged)
        self.style3D.AddObserver("WorldViewChanged", onWorldViewChanged)

        # Grid lines
        self.geoGridData = vtkGeoGridData()
        self.renderer2D.AddActor2D(self.geoGridData.GetActor2D())
        self.renderer3D.AddActor(self.geoGridData.GetActor3D())
        self.style2D.GetTransformCollection().AddItem(self.geoGridData.GetTransform())

        # Coastlines
        self.coastLineData = vtkGeographyLineData()
        self.coastLineData.SetMaxLevel(2)
        self.renderer2D.AddActor2D(self.coastLineData.GetActor2D())
        self.renderer3D.AddActor(self.coastLineData.GetActor3D())
        self.style2D.GetTransformCollection().AddItem(self.coastLineData.GetTransform())

        # Political Borders
        self.politicalBorderData = vtkGeographyLineData()
        self.politicalBorderData.SetMaxLevel(1)
        self.renderer2D.AddActor2D(self.politicalBorderData.GetActor2D())
        self.renderer3D.AddActor(self.politicalBorderData.GetActor3D())
        self.style2D.GetTransformCollection().AddItem(self.politicalBorderData.GetTransform())

        # Plot Title
        self.titleMapper2D = vtk.vtkTextMapper()
        self.titleMapper2D.GetTextProperty().SetFontFamilyToArial()
        self.titleMapper2D.GetTextProperty().SetFontSize(int(14 * self.GetDPIScaleFactor()))
        self.titleMapper2D.GetTextProperty().SetColor(0, 0, 0)
        self.titleMapper2D.GetTextProperty().SetShadow(1)
        self.titleMapper2D.GetTextProperty().SetBold(1)
        self.titleMapper2D.GetTextProperty().SetItalic(1)
        self.titleMapper2D.GetTextProperty().SetJustificationToCentered()
        self.titleActor2D = vtk.vtkActor2D()
        self.titleActor2D.SetMapper(self.titleMapper2D)
        self.titleActor2D.GetPositionCoordinate().SetCoordinateSystemToNormalizedViewport()
        self.titleActor2D.SetPosition(0.5, 0.95)
        self.renderer2D.AddActor2D(self.titleActor2D)

        self.titleMapper3D = vtk.vtkTextMapper()
        self.titleMapper3D.GetTextProperty().SetFontFamilyToArial()
        self.titleMapper3D.GetTextProperty().SetFontSize(int(14 * self.GetDPIScaleFactor()))
        self.titleMapper3D.GetTextProperty().SetColor(1, 1, 1)
        self.titleMapper3D.GetTextProperty().SetShadow(1)
        self.titleMapper3D.GetTextProperty().SetBold(1)
        self.titleMapper3D.GetTextProperty().SetItalic(1)
        self.titleMapper3D.GetTextProperty().SetJustificationToCentered()
        titleActor = vtk.vtkActor2D()
        titleActor.SetMapper(self.titleMapper3D)
        titleActor.GetPositionCoordinate().SetCoordinateSystemToNormalizedViewport()
        titleActor.SetPosition(0.5, 0.95)
        self.renderer3D.AddActor2D(titleActor)

        # Color Bar
        self.colorBarRenderer = vtk.vtkRenderer()
        self.colorBarActor = vtk.vtkScalarBarActor()
        defaultLut = vtk.vtkLookupTable()
        defaultLut.SetNumberOfTableValues(1)
        defaultLut.SetTableValue(0, 0, 0, 0, 0)
        self.colorBarActor.SetLookupTable(defaultLut)
        self.colorBarActor.SetOrientationToHorizontal()
        self.colorBarActor.SetPosition(0.1, 0.1)
        self.colorBarActor.SetPosition2(0.8, 0.9)
        self.colorBarActor.SetNumberOfLabels(5)
        self.colorBarActor.GetLabelTextProperty().SetColor(1, 1, 1)
        self.colorBarActor.GetLabelTextProperty().ShadowOff()
        self.colorBarActor.GetLabelTextProperty().SetFontFamilyToArial()
        self.colorBarActor.GetLabelTextProperty().ItalicOff()
        self.colorBarActor.GetLabelTextProperty().BoldOff()
        self.colorBarActor.GetLabelTextProperty().SetJustificationToCentered()
        self.colorBarActor.GetTitleTextProperty().SetColor(1, 1, 1)
        self.colorBarActor.GetTitleTextProperty().ShadowOff()
        self.colorBarActor.GetTitleTextProperty().SetFontFamilyToArial()
        self.colorBarActor.GetTitleTextProperty().ItalicOff()
        self.colorBarActor.GetTitleTextProperty().BoldOff()
        self.colorBarActor.SetLabelFormat("%g")
        self.colorBarRenderer.AddActor2D(self.colorBarActor)
        self.colorBarRenderer.InteractiveOff()

        # Make sure the renderer starts with the default zoom/pan position from style3D.
        # We need to do this step before the renderer is attached to the renderwindow otherwise the
        # renderwindow will start to draw itself which breaks the wxVTK binding.
        self.style3D.SetCurrentRenderer(self.renderer3D)
        self.style3D.SetDefaultZoom(2.5)
        self.style3D.SetDefaultView()

        # start with 3D projection
        self.projection = PROJECTION_3D
        self.GetRenderWindow().AddRenderer(self.colorBarRenderer)
        self.GetRenderWindow().AddRenderer(self.renderer2D)
        self.GetRenderWindow().AddRenderer(self.renderer3D)
        self.renderer2D.DrawOff()
        self.SetInteractorStyle(self.style3D)

        # By default the colorbar is hidden
        self.showColorBar = False
        self.UpdateColorBarSize()

    def SetPlotTitle(self, title):
        self.titleMapper2D.SetInput(title)
        self.titleMapper3D.SetInput(title)
        self.Refresh()

    def GetPlotTitle(self):
        return self.titleMapper2D.GetInput()

    def AddWorldPlotData(self, data):
        self.dataSets.append(data)
        self.renderer2D.RemoveActor2D(self.titleActor2D)
        self.renderer2D.AddActor2D(data.GetActor2D())
        self.renderer2D.AddActor2D(self.titleActor2D)
        self.renderer3D.AddActor(data.GetActor3D())
        self.style2D.GetTransformCollection().AddItem(data.GetTransform())
        if len(self.dataSets) == 1:
            self.SetDataSetForColorBar(0)

        def onColorTableChanged(caller, event, calldata=None):
            self.Refresh()
        data.GetColorTable().AddObserver("ColorTableChanged", onColorTableChanged)

        self.SetProjection(self.projection)
        self.SetProjectionCenterLatitude(self.GetProjectionCenterLatitude())
        self.SetProjectionCenterLongitude(self.GetProjectionCenterLongitude())
        if self.projection != PROJECTION_3D:
            self.UpdateStyle2D()

        if data.GetNumberOfKeyframes() > self.numKeyframes:
            self.numKeyframes = data.GetNumberOfKeyframes()
        self.Refresh()

        wx.PostEvent(self, WorldPlotDataChangedEvent())

        return len(self.dataSets) - 1

    def AddPointData(self, latitude, longitude, data=None, dataSetId=None):
        if dataSetId is None:
            pointData = vtkWorldPlotPointData()
        else:
            pointData = self.dataSets[dataSetId]
            if pointData.__class__.__name__ != "vtkWorldPlotPointData":
                raise Exception("pointData can only be added to existing pointData")
        latitude = numpy_to_vtk(numpy.asarray(latitude, dtype=numpy.double))
        longitude = numpy_to_vtk(numpy.asarray(longitude, dtype=numpy.double))
        if data is not None:
            data = numpy_to_vtk(numpy.asarray(data, dtype=numpy.double))
        pointData.AddData(latitude, longitude, data)
        if dataSetId is None:
            return self.AddWorldPlotData(pointData)
        else:
            if pointData.GetNumberOfKeyframes() > self.numKeyframes:
                self.numKeyframes = pointData.GetNumberOfKeyframes()
            self.Refresh()
            wx.PostEvent(self, WorldPlotDataChangedEvent())
        return dataSetId

    def AddLineData(self, latitude, longitude, dataSetId=None):
        if dataSetId is None:
            lineData = vtkWorldPlotLineData()
        else:
            lineData = self.dataSets[dataSetId]
            if lineData.__class__.__name__ != "vtkWorldPlotLineData":
                raise Exception("lineData can only be added to existing lineData")
        latitude = numpy_to_vtk(numpy.asarray(latitude, dtype=numpy.double))
        longitude = numpy_to_vtk(numpy.asarray(longitude, dtype=numpy.double))
        lineData.AddData(latitude, longitude)
        if dataSetId is None:
            return self.AddWorldPlotData(lineData)
        else:
            if lineData.GetNumberOfKeyframes() > self.numKeyframes:
                self.numKeyframes = lineData.GetNumberOfKeyframes()
            self.Refresh()
            wx.PostEvent(self, WorldPlotDataChangedEvent())
        return dataSetId

    def AddSwathData(self, cornerLatitude, cornerLongitude, data=None, dataSetId=None):
        if dataSetId is None:
            swathData = vtkWorldPlotSwathData()
        else:
            swathData = self.dataSets[dataSetId]
            if swathData.__class__.__name__ != "vtkWorldPlotSwathData":
                raise Exception("swathData can only be added to existing swathData")
        cornerLatitude = numpy_to_vtk(numpy.asarray(cornerLatitude, dtype=numpy.double))
        cornerLongitude = numpy_to_vtk(numpy.asarray(cornerLongitude, dtype=numpy.double))
        if data is not None:
            data = numpy_to_vtk(numpy.asarray(data, dtype=numpy.double))
        swathData.AddData(cornerLatitude, cornerLongitude, data)
        if dataSetId is None:
            return self.AddWorldPlotData(swathData)
        else:
            if swathData.GetNumberOfKeyframes() > self.numKeyframes:
                self.numKeyframes = swathData.GetNumberOfKeyframes()
            self.Refresh()
            wx.PostEvent(self, WorldPlotDataChangedEvent())
        return dataSetId

    def AddGridData(self, latitude, longitude, data, dataSetId=None):
        if dataSetId is None:
            gridData = vtkWorldPlotGridData()
        else:
            gridData = self.dataSets[dataSetId]
            if gridData.__class__.__name__ != "vtkWorldPlotGridData":
                raise Exception("gridData can only be added to existing gridData")
        latitude = numpy_to_vtk(numpy.asarray(latitude, dtype=numpy.double))
        longitude = numpy_to_vtk(numpy.asarray(longitude, dtype=numpy.double))
        data = numpy_to_vtk(numpy.ravel(numpy.asarray(data, dtype=numpy.double)))
        gridData.AddData(latitude, longitude, data)
        if dataSetId is None:
            return self.AddWorldPlotData(gridData)
        else:
            if gridData.GetNumberOfKeyframes() > self.numKeyframes:
                self.numKeyframes = gridData.GetNumberOfKeyframes()
            self.Refresh()
            wx.PostEvent(self, WorldPlotDataChangedEvent())
        return dataSetId

    def GetNumKeyframesForDataSet(self, dataSetId):
        return self.dataSets[dataSetId].GetNumberOfKeyframes()

    def GetNumDataSets(self):
        return len(self.dataSets)

    def GetNumKeyframes(self):
        return self.numKeyframes

    def SetKeyframe(self, newKeyframe):
        if newKeyframe < 0:
            newKeyframe = 0
        if newKeyframe >= self.numKeyframes:
            newKeyframe = self.numKeyframes - 1
        if newKeyframe != self.keyframe:
            self.keyframe = newKeyframe
            for dataSet in self.dataSets:
                dataSet.SetKeyframe(self.keyframe)
        self.Refresh()

    def GetKeyframe(self):
        return self.keyframe

    def SetProjection(self, projection):
        if projection != self.projection:
            # prevent multiple renderings of the VTK window
            self.Freeze()

            if self.projection == PROJECTION_3D:
                # switch from 3D to 2D
                viewLat = self.style3D.GetViewCenterLatitude()
                viewLon = self.style3D.GetViewCenterLongitude()
                zoom = self.style3D.GetViewZoom()
                self.renderer3D.DrawOff()
                self.renderer2D.DrawOn()
                self.SetInteractorStyle(self.style2D)
                self.colorBarRenderer.SetBackground(1, 1, 1)
                self.colorBarActor.GetLabelTextProperty().SetColor(0, 0, 0)
                self.colorBarActor.GetTitleTextProperty().SetColor(0, 0, 0)
            elif projection == PROJECTION_3D:
                # switch from 2D to 3D
                x = self.style2D.GetViewMidPointX()
                y = self.style2D.GetViewMidPointY()
                viewLat, viewLon = vtkProjFilter.NormalizedDeprojection2D(PROJECTION_IDS[self.projection],
                                                                          self.projCenterLatitude,
                                                                          self.projCenterLongitude, x, y)
                zoom = self.style2D.GetViewZoom()
                self.renderer2D.DrawOff()
                self.renderer3D.DrawOn()
                self.SetInteractorStyle(self.style3D)
                self.colorBarRenderer.SetBackground(0, 0, 0)
                self.colorBarActor.GetLabelTextProperty().SetColor(1, 1, 1)
                self.colorBarActor.GetTitleTextProperty().SetColor(1, 1, 1)
            else:
                # 2D to 2D
                x = self.style2D.GetViewMidPointX()
                y = self.style2D.GetViewMidPointY()
                viewLat, viewLon = vtkProjFilter.NormalizedDeprojection2D(PROJECTION_IDS[self.projection],
                                                                          self.projCenterLatitude,
                                                                          self.projCenterLongitude, x, y)
                zoom = self.style2D.GetViewZoom()

            self.geoGridData.SetProjection(PROJECTION_IDS[projection])
            self.coastLineData.SetProjection(PROJECTION_IDS[projection])
            self.politicalBorderData.SetProjection(PROJECTION_IDS[projection])
            for dataSet in self.dataSets:
                dataSet.SetProjection(PROJECTION_IDS[projection])

            self.projection = projection

            # set the view parameters
            if self.projection == PROJECTION_3D:
                self.style3D.SetViewParameters(viewLat, viewLon, 0.0, zoom)
            else:
                x, y = vtkProjFilter.NormalizedProjection2D(PROJECTION_IDS[self.projection], self.projCenterLatitude,
                                                            self.projCenterLongitude, viewLat, viewLon)

                self.UpdateStyle2D()

                self.style2D.SetViewMidPoint(x, y)
                self.style2D.SetViewZoom(zoom)
                if self.projection in CYLINDRICAL_PROJECTIONS:
                    self.style2D.SetDefaultZoom(1.6)
                else:
                    self.style2D.SetDefaultZoom(1.0)

            # allow rendering of the VTK window again
            self.Thaw()
            self.Refresh()

    def GetProjection(self):
        return self.projection

    def SetCoastLineFile(self, filename):
        self.coastLineData.SetFileName(filename)
        self.Refresh()

    def GetCoastLineFile(self):
        return self.coastLineData.GetFileName()

    def SetPoliticalBorderFile(self, filename):
        self.politicalBorderData.SetFileName(filename)
        self.Refresh()

    def GetPoliticalBorderFile(self):
        return self.politicalBorderData.GetFileName()

    def GetColorTable(self, dataSetId):
        return self.dataSets[dataSetId].GetColorTable()

    def ShowColorBar(self, show):
        if show != self.showColorBar:
            self.showColorBar = show
            self.UpdateColorBarSize()
            if self.projection != PROJECTION_3D:
                self.UpdateStyle2D()
            self.Refresh()

    def SetDataSetForColorBar(self, dataSetId):
        if dataSetId != self.dataSetForColorBar:
            self.dataSetForColorBar = dataSetId
            self.colorBarActor.SetLookupTable(self.dataSets[dataSetId].GetColorTable().GetVTKLookupTable())
            self.colorBarActor.SetTitle(self.dataSets[dataSetId].GetColorBarTitle())
            self.colorBarActor.SetNumberOfLabels(self.dataSets[dataSetId].GetNumColorBarLabels())
            self.UpdateColorBarSize()
            self.Refresh()

    def GetDataSetForColorBar(self):
        return self.dataSetForColorBar

    def SetProjectionCenterLatitude(self, projCenterLatitude):
        # This is NOT relevant for 3D
        if projCenterLatitude != self.projCenterLatitude:
            viewCenterLatitude = self.GetViewCenterLatitude()
            viewCenterLongitude = self.GetViewCenterLongitude()
            self.projCenterLatitude = projCenterLatitude
            self.geoGridData.SetProjectionCenterLatitude(projCenterLatitude)
            self.coastLineData.SetProjectionCenterLatitude(projCenterLatitude)
            self.politicalBorderData.SetProjectionCenterLatitude(projCenterLatitude)
            for dataSet in self.dataSets:
                dataSet.SetProjectionCenterLatitude(projCenterLatitude)
            self.SetViewCenter(viewCenterLatitude, viewCenterLongitude)
            self.Refresh()

    def GetProjectionCenterLatitude(self):
        return self.projCenterLatitude

    def SetProjectionCenterLongitude(self, projCenterLongitude):
        # This is NOT relevant for 3D
        if projCenterLongitude != self.projCenterLongitude:
            viewCenterLatitude = self.GetViewCenterLatitude()
            viewCenterLongitude = self.GetViewCenterLongitude()
            self.projCenterLongitude = projCenterLongitude
            self.geoGridData.SetProjectionCenterLongitude(projCenterLongitude)
            self.coastLineData.SetProjectionCenterLongitude(projCenterLongitude)
            self.politicalBorderData.SetProjectionCenterLongitude(projCenterLongitude)
            for dataSet in self.dataSets:
                dataSet.SetProjectionCenterLongitude(projCenterLongitude)
            self.SetViewCenter(viewCenterLatitude, viewCenterLongitude)
            self.Refresh()

    def GetProjectionCenterLongitude(self):
        return self.projCenterLongitude

    def SetViewCenter(self, viewCenterLatitude, viewCenterLongitude):
        # Pass through to the appropriate interactor style
        if self.projection == PROJECTION_3D:
            self.style3D.SetLatitude(viewCenterLatitude)
            self.style3D.SetLongitude(viewCenterLongitude)
        else:
            # modify the mid-point by mapping the view coordinates
            x, y = vtkProjFilter.NormalizedProjection2D(PROJECTION_IDS[self.projection], self.projCenterLatitude,
                                                        self.projCenterLongitude, viewCenterLatitude,
                                                        viewCenterLongitude)
            self.style2D.SetViewMidPoint(x, y)
            self.Refresh()

    def SetViewZoom(self, zoomScale):
        if self.projection == PROJECTION_3D:
            self.style3D.SetZoom(zoomScale)
        else:
            self.style2D.SetViewZoom(zoomScale)

    def GetViewCenterLatitude(self):
        if self.projection == PROJECTION_3D:
            return self.style3D.GetViewCenterLatitude()
        else:
            x = self.style2D.GetViewMidPointX()
            y = self.style2D.GetViewMidPointY()
            return vtkProjFilter.NormalizedDeprojection2D(PROJECTION_IDS[self.projection], self.projCenterLatitude,
                                                          self.projCenterLongitude, x, y)[0]

    def GetViewCenterLongitude(self):
        if self.projection == PROJECTION_3D:
            return self.style3D.GetViewCenterLongitude()
        else:
            x = self.style2D.GetViewMidPointX()
            y = self.style2D.GetViewMidPointY()
            return vtkProjFilter.NormalizedDeprojection2D(PROJECTION_IDS[self.projection], self.projCenterLatitude,
                                                          self.projCenterLongitude, x, y)[1]

    def GetViewZoom(self):
        if self.projection == PROJECTION_3D:
            return self.style3D.GetViewZoom()
        else:
            return self.style2D.GetViewZoom()

    def SetColorRange(self, dataSetId, minValue, maxValue):
        self.dataSets[dataSetId].GetColorTable().SetColorRange(minValue, maxValue)
        self.Refresh()

    def GetColorRange(self, dataSetId):
        return self.dataSets[dataSetId].GetColorTable().GetColorRange()

    def SetColorBarTitle(self, dataSetId, title):
        self.dataSets[dataSetId].SetColorBarTitle(title)
        if dataSetId == self.dataSetForColorBar:
            self.colorBarActor.SetLookupTable(self.dataSets[dataSetId].GetColorTable().GetVTKLookupTable())
            self.colorBarActor.SetTitle(title)
            self.UpdateColorBarSize()
        self.Refresh()

    def GetColorBarTitle(self, dataSetId):
        return self.dataSets[dataSetId].GetColorBarTitle()

    def SetDataSetLabel(self, dataSetId, label):
        self.dataSets[dataSetId].SetPlotLabel(label)
        self.Refresh()

    def GetDataSetLabel(self, dataSetId):
        return self.dataSets[dataSetId].GetPlotLabel()

    def SetNumColorBarLabels(self, dataSetId, numLabels):
        self.dataSets[dataSetId].SetNumColorBarLabels(numLabels)
        if dataSetId == self.dataSetForColorBar:
            self.colorBarActor.SetNumberOfLabels(numLabels)
        self.Refresh()

    def GetNumColorBarLabels(self, dataSetId):
        return self.dataSets[dataSetId].GetNumColorBarLabels()

    def SetOpacity(self, dataSetId, opacity):
        self.dataSets[dataSetId].SetOpacity(opacity)
        self.Refresh()

    def GetOpacity(self, dataSetId):
        return self.dataSets[dataSetId].GetOpacity()

    def SetLineWidth(self, dataSetId, width):
        self.dataSets[dataSetId].SetLineWidth(width)
        self.Refresh()

    def GetLineWidth(self, dataSetId):
        return self.dataSets[dataSetId].GetLineWidth()

    def SetPointSize(self, dataSetId, size):
        self.dataSets[dataSetId].SetPointSize(size)
        self.Refresh()

    def GetPointSize(self, dataSetId):
        return self.dataSets[dataSetId].GetPointSize()

    def SetReferenceHeight(self, dataSetId, referenceHeight):
        self.dataSets[dataSetId].SetReferenceHeight(referenceHeight)
        self.Refresh()

    def GetReferenceHeight(self, dataSetId):
        return self.dataSets[dataSetId].GetReferenceHeight()

    def SetHeightFactor(self, dataSetId, heightFactor):
        self.dataSets[dataSetId].SetHeightFactor(heightFactor)
        self.Refresh()

    def GetHeightFactor(self, dataSetId):
        return self.dataSets[dataSetId].GetHeightFactor()

    def SetMinHeightValue(self, dataSetId, minValue):
        self.dataSets[dataSetId].SetMinHeightValue(minValue)
        self.Refresh()

    def GetMinHeightValue(self, dataSetId):
        return self.dataSets[dataSetId].GetMinHeightValue()

    def SetMaxHeightValue(self, dataSetId, maxValue):
        self.dataSets[dataSetId].SetMaxHeightValue(maxValue)
        self.Refresh()

    def GetMaxHeightValue(self, dataSetId):
        return self.dataSets[dataSetId].GetMaxHeightValue()

    def ExportToImageFile(self, filename, format):
        if format == "tif":
            writer = vtk.vtkTIFFWriter()
        elif format == "bmp":
            writer = vtk.vtkBMPWriter()
        elif format == "jpg":
            writer = vtk.vtkJPEGWriter()
        elif format == "png":
            writer = vtk.vtkPNGWriter()
        elif format == "pnm":
            writer = vtk.vtkPNMWriter()
        else:
            raise Exception("Unsupported export format")
        imageFilter = vtk.vtkWindowToImageFilter()
        imageFilter.SetInput(self.GetRenderWindow())
        writer.SetInputConnection(imageFilter.GetOutputPort())
        writer.SetFileName(filename)
        self.GetRenderWindow().Render()
        writer.Write()

    def OnSize(self, event):
        self.UpdateColorBarSize()
        if self.projection != PROJECTION_3D:
            self.UpdateStyle2D()
        event.Skip()

    def UpdateStyle2D(self):
        # Update 2D interactor style with new window size
        w, h = self.GetSize()
        if self.showColorBar:
            h -= self.colorBarHeight
        ratio = self.geoGridData.GetXYRatio()
        self.style2D.SetViewportSizeAndDataXYRatio(w, h, ratio)

    def UpdateColorBarSize(self):
        if self.showColorBar:
            width, height = self.GetSize()
            if height < 1:
                height = 1
            if not self.colorBarActor.GetTitle():
                relativeHeight = float(self.colorBarHeight) * 3.0 / (height * 4.0)
                self.colorBarActor.SetBarRatio(0.65)
            else:
                relativeHeight = float(self.colorBarHeight) / height
                self.colorBarActor.SetBarRatio(0.5)
        else:
            relativeHeight = 0.0
        self.colorBarRenderer.SetViewport(0, 0, 1, relativeHeight)
        self.renderer2D.SetViewport(0, relativeHeight, 1, 1)
        self.renderer3D.SetViewport(0, relativeHeight, 1, 1)



# This is an example VISAN script for the MIP_NL__1P product

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing MIP_NL__1P products.

# This example will then take the first product it finds in this directory and
# for that product plot the measured fourier spectra for band AB


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all files in productdir starting with 'MIP_NL__1P'
    files = glob.glob(os.path.join(productdir, "MIP_NL__1P*"))

    if len(files) == 0:
        print(("Could not find any MIP_NL__1P files in directory '" + productdir + "'"))
        return

    # We only ingest the first file from the list
    product = harp.import_product(files[0], options="band=AB")

    plot(product, ylabel="W/(cm^2\n sr cm^-1)", showpropertypanel=True, name=os.path.basename(files[0]),
         title="MIP_NL__1P spectral readout example (band AB)")


run()



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import wx


class FloatValidator(wx.Validator):

    def __init__(self):
        super(FloatValidator, self).__init__()
        self.Bind(wx.EVT_CHAR, self.OnChar)
        self.floatchars = '0123456789-+.eE'

    def Clone(self):
        return FloatValidator()

    def OnChar(self, event):
        key = event.KeyCode
        if key < wx.WXK_SPACE or key == wx.WXK_DELETE or key > 255:
            event.Skip()
            return

        if chr(key) in self.floatchars:
            event.Skip()
            return

        if not wx.Validator_IsSilent():
            wx.Bell()

        return


class DigitValidator(wx.Validator):

    def __init__(self):
        super(DigitValidator, self).__init__()
        self.Bind(wx.EVT_CHAR, self.OnChar)
        self.digits = '0123456789'

    def Clone(self):
        return DigitValidator()

    def OnChar(self, event):
        key = event.KeyCode
        if key < wx.WXK_SPACE or key == wx.WXK_DELETE or key > 255:
            event.Skip()
            return

        if chr(key) in self.digits:
            event.Skip()
            return

        if not wx.Validator_IsSilent():
            wx.Bell()

        return



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import wx
import wx.lib.newevent

from .datasetattributespanel import DataSetAttributesPanel
from .geolocationpanel import GeolocationPanel
from .util import DetermineCharSize

CurrentDataSetChangedEvent, EVT_CURRENTDATASET_CHANGED = wx.lib.newevent.NewEvent()


class PlotDataSetPanel(wx.Panel):

    def __init__(self, parent, plotFrame):
        panelstyle = wx.TAB_TRAVERSAL
        if wx.Platform == '__WXGTK__':
            panelstyle |= wx.SUNKEN_BORDER
        wx.Panel.__init__(self, parent, -1, style=panelstyle)

        # Set initial state
        self.plotFrame = plotFrame

        # Create and configure all widgets
        self.CreateControls()
        self.CreateLayout()

        self.UpdateDataSetList()

    def CreateControls(self):
        (charWidth, charHeight) = DetermineCharSize(self)

        self.dataSetChoice = wx.Choice(self, -1, size=(10 * charWidth, -1))
        self.dataSetChoice.Bind(wx.EVT_CHOICE, self.OnDataSetSelectionChanged)

        self.dataSetPropertiesNotebook = wx.Notebook(self, -1)
        self.dataSetAttributesPanel = DataSetAttributesPanel(self.dataSetPropertiesNotebook)
        self.dataSetPropertiesNotebook.AddPage(self.dataSetAttributesPanel, "Attributes")
        self.dataSetLocationPanel = GeolocationPanel(self.dataSetPropertiesNotebook)
        self.dataSetPropertiesNotebook.AddPage(self.dataSetLocationPanel, "Location")

    def CreateLayout(self):
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.dataSetChoice, 0, wx.EXPAND | wx.ALL, border=10)
        sizer.Add(self.dataSetPropertiesNotebook, 1, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.BOTTOM, border=2)
        self.SetSizer(sizer)

    def UpdateDataSetList(self):
        oldDataSetId = self.dataSetChoice.GetSelection()
        self.dataSetChoice.Clear()
        for i in range(self.plotFrame.GetNumDataSets()):
            name = str(self.plotFrame.GetDataSetLabel(i))
            if not name:
                name = 'dataset #%d' % (i + 1)
            self.dataSetChoice.Insert(name, self.dataSetChoice.GetCount())
        if self.dataSetChoice.GetCount() > 0:
            newDataSetId = oldDataSetId
            if newDataSetId < 0 or newDataSetId >= self.dataSetChoice.GetCount():
                newDataSetId = 0
            self.dataSetChoice.Select(newDataSetId)
            if newDataSetId != oldDataSetId:
                self.UpdateAttributes()
                self.UpdateLocation()

    def UpdateAttributes(self):
        dataSetId = self.dataSetChoice.GetSelection()
        if dataSetId >= 0:
            attributes = self.plotFrame.dataSetAttributes[dataSetId]
            keyframe = self.plotFrame.GetKeyframe()
            self.dataSetAttributesPanel.UpdateAttributes(attributes, keyframe)
            self.Update()

    def UpdateLocation(self):
        dataSetId = self.dataSetChoice.GetSelection()
        if dataSetId >= 0 and not self.plotFrame.closingDown:
            location = self.plotFrame.dataSetLocation[dataSetId]
            keyframe = self.plotFrame.GetKeyframe()
            self.dataSetLocationPanel.UpdateLocation(dataSetId, location, keyframe)
            self.Update()

    def OnDataSetSelectionChanged(self, event):
        self.UpdateAttributes()
        self.UpdateLocation()
        dataSetId = self.dataSetChoice.GetSelection()
        if dataSetId >= 0:
            wx.PostEvent(self, CurrentDataSetChangedEvent(dataSetId=dataSetId))



# This is an example VISAN script for the fit() function.

from visan.math import fit

def run():

    # Create some sample x and y data
    x = numpy.random.random(100) * 10.0
    y = numpy.random.random(100) * 10.0
    x = numpy.sort(x)
    y = numpy.sort(y)
    # We give our y values a standard deviation of 0.5
    weights = 0.5 * numpy.ones(numpy.shape(y))

    # Calculate the best guesses for a, b for the linear function y = a + b * x
    a, b, err = fit(x, y, sigy=weights, error=True)

    # Print a, b and the error information
    print(("a = %f, b = %f, siga = %f, sigb = %f, chi2 = %f, q = %f" %
          (a, b, err['siga'], err['sigb'], err['chi2'], err['q'])))

    # Plot the initial values and our fitted linear function
    w = plot(x, y, lines=0, points=1, name="samples")
    plot(x, a + b * x, xmin=-1, xmax=11, ymin=-1, ymax=11, name="fitted line", window=w, title="fit() example")


run()
